"""Import modules"""
# pylint: disable=R0903
import os
import sys
import slack
from dotenv import load_dotenv


class Slack():
    """Send message methods"""
    @classmethod
    def message(cls, message):
        """Send the output of Service to Slack"""
        load_dotenv()
        slack_client = slack.WebClient(os.getenv('SLACK_BOT_TOKEN'))
        try:
            slack_client.chat_postMessage(
                channel=os.getenv('SLACK_CHANNEL'),
                text=message
            )
            success = 'Slack message sent!'
            return success
        except slack.errors.SlackApiError:
            final_output = 'Error: could not send the Service Slack message.'
            sys.exit(final_output)



"""Import modules"""
# pylint: disable=R0903
import platform


class System():
    """System information"""
    @classmethod
    def serve(cls):
        """Serve system info"""
        # Title
        system_title = '='*15 + ' System Information ' + '='*15

        # System info
        uname = platform.uname()
        system_name = f'System: {uname.system}'
        node_name = f'Node Name: {uname.node}'
        release = f'Release: {uname.release}'
        version = f'Version: {uname.version}'
        machine = f'Machine: {uname.machine}'
        processor = f'Processor: {uname.processor}'

        final_message = '\n' + system_title + '\n' + system_name + '\n' + node_name + \
            '\n' + release + '\n' + version + '\n' + machine + '\n' + processor
        return final_message



"""Import Service modules"""
from .boot_time import Boot
from .cpu import Cpu
from .disk import Disk
from .globals import Global
from .memory import Memory
from .network import Network
from .slack import Slack
from .system import System



"""Import modules"""
# pylint: disable=R0903,R0914
import psutil
from .globals import Global


class Network():
    """Network information"""
    @classmethod
    def serve(cls):
        """Serve network info"""
        # Title
        network_title = '='*15 + ' Network Information ' + '='*15

        # Network information (virtual and physical)
        if_addrs = psutil.net_if_addrs()
        all_pieces = ''
        for interface_name, interface_addresses in if_addrs.items():
            for address in interface_addresses:
                interface_title = f'=== Interface: {interface_name} ==='
                if str(address.family) == 'AddressFamily.AF_INET':
                    ip_address = f'  IP Address: {address.address}'
                    netmask = f'  Netmask: {address.netmask}'
                    broadcast_ip = f'  Broadcast IP: {address.broadcast}'
                    # Combine each piece into a variable
                    piece = ip_address + '\n' + netmask + '\n' + broadcast_ip
                elif str(address.family) == 'AddressFamily.AF_PACKET':
                    mac = f'  MAC Address: {address.address}'
                    netmask = f'  Netmask: {address.netmask}'
                    broadcast_mac = f'  Broadcast MAC: {address.broadcast}'
                    # Combine each piece into a variable
                    piece = mac + '\n' + netmask + '\n' + broadcast_mac
                # Combine each piece into a variable
                all_pieces += interface_title + '\n' + piece + '\n'

        # get IO statistics since boot
        net_io = psutil.net_io_counters()
        sent = f'Total Bytes Sent: {Global.get_size(net_io.bytes_sent)}'
        received = f'Total Bytes Received: {Global.get_size(net_io.bytes_recv)}'

        final_message = '\n' + network_title + '\n' + \
            all_pieces + sent + '\n' + received
        return final_message



"""Import Modules"""
# pylint: disable=C0103,R0912
from datetime import datetime
import argparse
from .boot_time import Boot
from .cpu import Cpu
from .disk import Disk
from .memory import Memory
from .network import Network
from .slack import Slack
from .system import System


# Setup arguments
parser = argparse.ArgumentParser(
    description='Service serves savvy server stats.')
parser.add_argument('-b', '--boot', action='store_true',
                    help='Show boot time stats.')
parser.add_argument('-c', '--cpu', action='store_true',
                    help='Show CPU stats.')
parser.add_argument('-d', '--disk', action='store_true',
                    help='Show disk stats.')
parser.add_argument('-m', '--memory', action='store_true',
                    help='Show memory stats.')
parser.add_argument('-n', '--network', action='store_true',
                    help='Show network stats.')
parser.add_argument('-s', '--system', action='store_true',
                    help='Show system stats.')
parser.add_argument('-sl', '--slack', action='store_true',
                    help='Send Service report to Slack.')
args = parser.parse_args()


def main():
    """Run the Service app"""
    # Preamble
    service_message = '='*15 + ' SERVICE ' + '=' * \
        15 + '\n' + f'Service Report ({datetime.now()})\n{args}'
    print(service_message)

    # Serve boot time data
    if args.boot is True:
        boot = Boot.serve()
        print(boot)
    else:
        boot = ''

    # Serve CPU data
    if args.cpu is True:
        cpu = Cpu.serve()
        print(cpu)
    else:
        cpu = ''

    # Serve disk data
    if args.disk is True:
        disk = Disk.serve()
        print(disk)
    else:
        disk = ''

    # Serve memory data
    if args.memory is True:
        memory = Memory.serve()
        print(memory)
    else:
        memory = ''

    # Serve network data
    if args.network is True:
        network = Network.serve()
        print(network)
    else:
        network = ''

    # Serve system data
    if args.system is True:
        system = System.serve()
        print(system)
    else:
        system = ''

    # Send Slack message
    if args.slack is True:
        final_message = '\n' + service_message + '\n' + boot + '\n' + \
            system + '\n' + cpu + '\n' + disk + '\n' + memory + '\n' + network
        slack = Slack.message(final_message)
        print(slack)


if __name__ == '__main__':
    main()



"""Import modules"""
# pylint: disable=R0903
from datetime import datetime
import psutil


class Boot():
    """Boot time information"""
    @classmethod
    def serve(cls):
        """Serve boot time info"""
        # Title
        boot_title = '='*15 + ' Boot Time ' + '='*15

        # Timestamp
        boot_time_timestamp = psutil.boot_time()
        boot_time = datetime.fromtimestamp(boot_time_timestamp)
        boot_message = f'Boot Time: {boot_time.year}/{boot_time.month}/{boot_time.day}' \
            f'{boot_time.hour}:{boot_time.minute}:{boot_time.second}'

        final_message = '\n' + boot_title + '\n' + boot_message
        return final_message
